;;
;; Relocating ABC80 .bac loader loading at BOFA
;;
;; To load a device driver, move BOFA to the desired address
;; after initialization, move EOFA to the same address as BOFA
;; and put a 01h byte at the new EOFA (this does a NEW).
;;

	include "setup80.inc"

_start:
	pop hl			; Return address
	pop hl			; P-code position
	inc hl			; Skip end of integer statement
	inc hl			; Skip string constant opcode
	inc hl			; Skip string constant quotation mark
	ld c,(hl)		; String length
	inc hl
	;; HL = string
	;; C = string length
	inc c	; Copy the statement trailer too for simplicity
	inc c
	;; Make space on the stack
	ex de,hl
	xor a
	ld b,a
	ld l,a
	ld h,a
	sbc hl,bc
	add hl,sp
	ld sp,hl		; Make space for chain string on stack
	ex de,hl
	push bc			; String length
	push de			; String address
	ld a,(hl)		; First character in chain string
	ldir
	push bc			; Dummy argument (=0)
	ld c,END
	cp a,':'		; Bogus name -> END, not CHAIN ""
	jr z,gotretn
	ld bc,XCHAINo
	ld a,(bc)
	cp 0CDh			; Old BASIC?
	jr z,gotretn
	ld c,XCHAINn & 0xff	; No, it's new BASIC
gotretn:
	push bc			; Return address

	ld de,(BOFA)		; BOFA = start of data
	push de			; Save for done
	push de			; Save for relocation

	include "copy80.inc"

set_reloc_len:
	defc NEG_RELOC_LEN=set_reloc_len+1
	ld hl,0			; Negative (length of relocation data + 1)
	add hl,de

	;;
	;; Move EOFA to the end of our data
	;; This is probably unnecessary but easy to do
	;; The C code will already have inserted a 01h byte to this address
	;;
	ld (EOFA),hl
	inc hl			; HL -> relocations
	pop de			; DE -> data (= BOFA)

	;; Set a delta if needed.  This loader only supports an
	;; 8-bit delta
	ld a,e
setdelta:
	defc DELTA=setdelta+1
	add a,0
	ld c,a
	ld a,d
	adc a,0
	ld b,a

	include "relocate.inc"

	;; All done, jump to entry point
	;; The stack is all set up for return to CHAIN or END
done:
	defc ENTRY=done+1
	ld bc,0
	pop hl			; HL -> load address (= BOFA)
	add hl,bc
	jp (hl)

	include "end80.inc"
